'use strict';
/**
 * node-windows-window-controller
 *
 * @author waiting
 * @license MIT
 * @link https://github.com/waitingsong/node-windows-window-controller
 */
Object.defineProperty(exports, '__esModule', { value: true });
const ref = require('ref');
const win32_api_1 = require('win32-api');
const u32 = require('./lib/user32');
var user32_1 = require('./lib/user32');
exports.validate_cmdshow = user32_1.validate_cmdshow;
const isWin32 = process.platform === 'win32';
const plateformError = 'Invalid platform: win32 required';
const user32 = win32_api_1.U.load();
// expose original user32.ShowWindow() as default
function showWindow(hwnd, nCmdShow) {
    let errcode = 1;
    if (!Number.isSafeInteger(hwnd)) {
        console.error('hwnd must integer');
        return Promise.resolve(errcode);
    }
    if (!u32.validate_cmdshow(nCmdShow)) {
        return Promise.resolve(errcode);
    }
    return retrieve_pointer_by_hwnd(hwnd)
        .then((arr) => {
        if (arr && arr.length && !ref.isNull(arr[0])) {
            if (user32.ShowWindow(arr[0], nCmdShow)) {
                errcode = 0;
            }
        }
        return errcode;
    })
        .catch((err) => {
        return errcode;
    });
}
exports.default = showWindow;
function hide(p, onlyMainWin = true) {
    return proxy(p, 0 /* SW_HIDE */, onlyMainWin);
}
exports.hide = hide;
function show(p, nCmdShow, onlyMainWin = true) {
    return proxy(p, nCmdShow, onlyMainWin);
}
exports.show = show;
// https://msdn.microsoft.com/en-us/library/windows/desktop/ms633548(v=vs.85).aspx
function proxy(p, nCmdShow, onlyMainWin) {
    const execRet = init_execret();
    if (!u32.validate_cmdshow(nCmdShow)) {
        execRet.err = 1;
        execRet.msg = 'value of nCmdShow invalid';
        return Promise.resolve(execRet);
    }
    return new Promise(resolve => {
        if (onlyMainWin) {
            get_main_hwnd(p).then(hWnds => {
                if (hWnds && hWnds.length) {
                    for (const hWnd of hWnds) {
                        if (hWnd && !ref.isNull(hWnd)) {
                            // console.log('hWnd add:', ref.address(hWnd));
                            u32.show(hWnd, nCmdShow, onlyMainWin)
                                .then((hWnd) => {
                                hWnd && !ref.isNull(hWnd) && execRet.hwnds.push(ref.address(hWnd));
                            })
                                .catch(err => {
                                execRet.err = 1;
                                execRet.msg += '\n ' + err;
                            })
                                .then(() => {
                                resolve();
                            });
                        }
                    }
                }
                else {
                    resolve();
                }
            });
        }
        else {
            get_hwnds(p).then(arr => {
                if (arr && arr.length) {
                    Promise.all(arr.map(hWnd => {
                        return u32.show(hWnd, nCmdShow, onlyMainWin)
                            .then((hWnd) => hWnd && execRet.hwnds.push(ref.address(hWnd)))
                            .catch((err) => {
                            execRet.err = 1;
                            execRet.msg += '\n ' + err;
                        });
                    }))
                        .then(resolve);
                }
                else {
                    resolve();
                }
            });
        }
    }).then(() => execRet);
}
function get_main_hwnd(p) {
    return get_hwnds(p).then((arr) => {
        if (arr && Array.isArray(arr) && arr.length) {
            return u32.filter_main_hwnd(arr);
        }
    });
}
exports.get_main_hwnd = get_main_hwnd;
function get_hwnds(p) {
    return u32.get_hwnds(p);
}
exports.get_hwnds = get_hwnds;
// kill process by which the matched hWnd(s) (window) created
function kill(p) {
    const execRet = init_execret();
    if (typeof p === 'number') {
        _kill(p);
        execRet.pids.push(p);
        return Promise.resolve(execRet);
    }
    else if (typeof p === 'string') {
        return new Promise(resolve => {
            const task = u32.create_task();
            return u32.get_hwnds(p, task).then(() => {
                if (task.pidSet.size) {
                    for (let pid of task.pidSet) {
                        if (_kill(pid)) {
                            execRet.pids.push(pid);
                        }
                    }
                }
                else {
                    execRet.msg = 'the pid list to be killed empty';
                }
                resolve(execRet);
            });
        });
    }
    else {
        execRet.err = 1;
        execRet.msg = 'kill() param must be either number or string';
        return Promise.resolve(execRet);
    }
}
exports.kill = kill;
function _kill(pid) {
    try {
        process.kill(pid, 0) && process.kill(pid);
        return true;
    }
    catch (ex) {
        console.error(ex);
    }
    return false;
}
function init_execret() {
    return {
        err: 0,
        msg: '',
        pids: [],
        hwnds: [],
    };
}
// retrive hWnd buffer from decimal/hex value of hWnd
function retrieve_pointer_by_hwnd(hwnd) {
    const task = u32.create_task();
    task.matchType = 'hwnd';
    return u32.get_hwnds(hwnd, task);
}
exports.retrieve_pointer_by_hwnd = retrieve_pointer_by_hwnd;
