'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
const ffi = require('ffi');
const ref = require('ref');
const win32_api_1 = require('win32-api');
const user32 = win32_api_1.U.load();
const isWin32 = process.platform === 'win32';
const plateformError = 'Invalid platform: win32 required';
exports.taskConfig = {
    tno: 0,
    task: new Map(),
};
// stop loop if return false
exports.enumWindowsProc = ffi.Callback(win32_api_1.windef.BOOL, [win32_api_1.windef.HWND, win32_api_1.windef.LPARAM], (hWnd, lParam) => {
    const task = exports.taskConfig.task.get(lParam);
    if (!task) {
        return true;
    }
    switch (task.matchType) {
        case 'pid': {
            const buf = ref.alloc(win32_api_1.windef.HINSTANCE);
            user32.GetWindowThreadProcessId(hWnd, buf);
            const pid = buf.readUInt32LE(0);
            if (pid && pid === task.matchValue) {
                task.hwndSet.add(hWnd);
            }
            break;
        }
        case 'title':
            if (task.matchValue) {
                const buf = Buffer.alloc(254);
                const ret = user32.GetWindowTextW(hWnd, buf, 254);
                const name = buf.toString('ucs2');
                // const visible = user32.IsWindowVisible(hWnd);
                if (name.indexOf(task.matchValue) !== -1) {
                    const buf = Buffer.alloc(4);
                    user32.GetWindowThreadProcessId(hWnd, buf);
                    task.pidSet.add(buf.readUIntLE(0, 4));
                    task.hwndSet.add(hWnd);
                }
            }
            else {
                task.hwndSet.add(hWnd);
            }
            break;
        case 'hwnd':
            if (+task.matchValue === ref.address(hWnd)) {
                task.hwndSet.add(hWnd);
            }
            break;
        default:
            return true;
    }
    return true;
});
function validate_cmdshow(nCmdShow) {
    const res = (nCmdShow < 0) ? false : true;
    if (!res) {
        console.error('hWnd value invalid: See: https://msdn.microsoft.com/en-us/library/windows/desktop/ms633548(v=vs.85).aspx');
    }
    return res;
}
exports.validate_cmdshow = validate_cmdshow;
function show(hWnd, nCmdShow, onlyMainWin) {
    return new Promise((resolve, reject) => {
        if (onlyMainWin && !is_main_window(hWnd)) {
            return resolve();
        }
        nCmdShow = +nCmdShow;
        if (Number.isNaN(nCmdShow) || nCmdShow < 0) {
            return reject('show() params nCmdShow invalid: ' + nCmdShow);
        }
        if (nCmdShow === 0) {
            if (!user32.IsWindowVisible(hWnd)) {
                return resolve();
            }
        }
        try {
            user32.ShowWindow(hWnd, nCmdShow);
        }
        catch (ex) {
            return reject(ex);
        }
        resolve(hWnd);
    });
}
exports.show = show;
function get_hwnds(p, task) {
    if (!task) {
        task = create_task();
    }
    task.matchValue = p;
    let t;
    return Promise.race([
        new Promise(resolve => {
            t = setTimeout(() => {
                console.error('timeout failed');
                resolve();
            }, 10000); // @HARDCOD
        }),
        _get_hwnds(task),
    ]).then((res) => {
        clearTimeout(t);
        task && exports.taskConfig.task.delete(task.tno);
        if (res && Array.isArray(res) && res.length) {
            return res;
        }
    }).catch(err => {
        clearTimeout(t);
        task && exports.taskConfig.task.delete(task.tno);
        console.error(err);
    });
}
exports.get_hwnds = get_hwnds;
function _get_hwnds(task) {
    if (task.matchType === null) {
        if (typeof task.matchValue === 'string') {
            task.matchType = 'title';
        }
        else if (typeof task.matchValue === 'number') {
            task.matchType = 'pid';
        }
    }
    switch (task.matchType) {
        case 'pid':
            if (!Number.isSafeInteger(task.matchValue)) {
                return Promise.resolve();
            }
            break;
        case 'title':
            // void
            break;
        case 'hwnd':
            // void
            break;
        default:
            throw new Error('task.matchType value invalid');
    }
    return get_task_hwnd(task)
        .then(res => {
        return res;
    })
        .catch(err => {
        console.error(err);
        return;
    });
}
function get_task_hwnd(task) {
    return new Promise((resolve, reject) => {
        if (!isWin32) {
            reject(plateformError);
            return;
        }
        if (!task) {
            return resolve([]);
        }
        user32.EnumWindows.async(exports.enumWindowsProc, task.tno, (err) => {
            resolve(Array.from(task.hwndSet) || []);
        });
    });
}
// get hWnd of main top-level window
function filter_main_hwnd(arr) {
    const res = new Set();
    const ids = new Set();
    // console.log('filter_main_hwnd:', arr.length, ref.address(arr[0]));
    for (const hWnd of arr) {
        // const ownerHWnd = user32.GetAncestor(hWnd, 3);
        // if (! ownerHWnd || ref.isNull(ownerHWnd) || ref.address(ownerHWnd) === ref.address(hWnd)) {  // hWnd is top window
        //     ids.add(hWnd);
        // }
        if (is_main_window(hWnd)) {
            ids.add(hWnd);
        }
    }
    if (ids.size === 1) {
        const [hWnd] = ids.keys();
        return [hWnd];
    }
    else if (ids.size > 1) {
        return [...ids.keys()];
    }
}
exports.filter_main_hwnd = filter_main_hwnd;
// check hWnd is own window not child window
function is_main_window(hWnd) {
    const p = user32.GetParent(hWnd);
    return !p || ref.isNull(p) ? true : false;
}
exports.is_main_window = is_main_window;
function create_task() {
    const tno = ++exports.taskConfig.tno;
    const task = {
        tno,
        matchType: null,
        matchValue: '',
        hwndSet: new Set(),
        pidSet: new Set(),
    };
    exports.taskConfig.task.set(tno, task);
    return task;
}
exports.create_task = create_task;
